--------------------------------------------------------------------------------
-- Title   : Top - duplication function
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Top sheet of the duplication block
--    inTTL(0) is duplicated to outTTL[4 donwto 1] and outPECL[4 downto 1]
--    outTTL(0) is not used as the board IO connector is hard configured as
--    input
--------------------------------------------------------------------------------
-- File     : dup_top.vhdl
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : March 13, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the Lesser GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity dup_top is
  port (
    p_dup_inTTL             : in  std_logic;                     -- input TTL
    p_dup_outTTL            : out std_logic_vector (4 downto 0); -- TTL outputs
    p_dup_outPECL           : out std_logic_vector (4 downto 0); -- LVPECL outputs
    p_dup_clk1kHz           : in  std_logic;
    p_dup_clk500mHz         : in  std_logic;
    p_dup_clk60MHz          : in  std_logic;
    p_dup_missingPulseDelay : in  std_logic_vector (31 downto 0);
    p_dup_csw               : in  std_logic;
    p_dup_reset             : in  std_logic;
    p_dup_pulseMon          : out std_logic_vector (31 downto 0); -- pulses monitor
    p_dup_led               : out std_logic_vector (1 downto 0)
  );
end entity dup_top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_dup_top of dup_top is

  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  signal s_impulse : std_logic; -- detection of impulse

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

    -- Signal duplication block (either TTL or LVPECL)
--    duplication : entity work.dup_duplication (rtl_dup_duplicationPECL)
    duplication : entity work.dup_duplication (rtl_dup_duplicationTTL)
      port map (
        p_inTTL   => p_dup_inTTL,  -- TTL in
        p_outTTL  => p_dup_outTTL, -- TTL out
        p_outPECL => p_dup_outPECL -- LVPECL out
      );

    -- Signal monitoring block
    -- The time betwwen 2 input pulses in stored in MONITOR register (in ms)
    pulseMonitor : entity work.dup_monitoring (rtl_dup_monitoring)
      port map (
        p_inTTL   => p_dup_inTTL,    -- TTL input
        p_clk1kHz => p_dup_clk1kHz,
        p_reset   => p_dup_reset,
        p_monitor => p_dup_pulseMon  -- register with the time value between two pulses
      );

    -- LEDs management for the duplication block
    -- The green LED is blinked for 5s after a reset or after a input pulse is
    -- dtected. It remains ON in other cases.
    -- The red LED is off after a reset. It gos ON if no input signal is detected
    -- after delay defined in the missingPulseDelay register.
    leds : entity work.dup_leds (rtl_dup_leds)
      port map (
        p_inTTL             => p_dup_inTTL,  -- TTL input
        p_clk1kHz           => p_dup_clk1kHz,
        p_clk500mHz         => p_dup_clk500mHz,
        p_clk60MHz          => p_dup_clk60MHz,
        p_missingPulseDelay => p_dup_missingPulseDelay,
        p_csw               => p_dup_csw,
        p_reset             => p_dup_reset,
        p_led               => p_dup_led
      );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------

end architecture rtl_dup_top;
