--------------------------------------------------------------------------------
-- Title   : Top
-- Project : TimEX3
--------------------------------------------------------------------------------
-- Description : Top sheet
--------------------------------------------------------------------------------
-- File     : top.vhd
-- Revision : x.x.x
-- Created  : October 26, 2012
-- Updated  : April 12, 2013
--------------------------------------------------------------------------------
-- Author       : Jean-Paul Ricaud
-- Organization : Synchrotron Soleil
-- Web          : http://www.synchrotron-soleil.fr
-- Email        : jean-paul.ricaud@synchrotron-soleil.fr
--------------------------------------------------------------------------------
-- Copyright (C) 2012 - 2013 Synchrotron Soleil
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU Lesser General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-- GNU Lesser General Public License for more details.
--
-- You should have received a copy of the GNU Lesser General Public License
-- along with this program. If not, see <http://www.gnu.org/licenses/>
--------------------------------------------------------------------------------
--
--------------------------------------------------------------------------------
-- Modifications :
--
--------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

library unisim;
use unisim.vcomponents.all;

--------------------------------------------------------------------------------
--------------------------------- ENTITY ---------------------------------------
--------------------------------------------------------------------------------
entity top is
  port (
    -- PLX bus
    pin_AD       : in    std_logic_vector (9 downto 2);  -- address bus
    pin_DATA     : inout std_logic_vector (31 downto 0); -- data bus
    pin_CSn      : in    std_logic_vector (1 downto 0);  -- chip select
    pin_BEn      : in    std_logic_vector (3 downto 0);  -- byte enable
    pin_BLASTn   : in    std_logic; -- burst last
    pin_LWRn     : in    std_logic; -- read/write ; = LW/R
    pin_RDn      : in    std_logic; -- read strobe
    pin_WRn      : in    std_logic; -- write strobe ; = WR
    pin_WAITOn   : in    std_logic; -- wait
    pin_ADSn     : in    std_logic; -- address strobe
    pin_ALE      : in    std_logic; -- address latch
    pin_CLK60MHz : in    std_logic; -- clock 60 MHz
    pin_READYn   : out   std_logic; -- ready
    pin_BTERMn   : out   std_logic; -- burst treminate
    pin_LINTI    : out   std_logic_vector (1 downto 0); -- interrupt
    pin_RESETn   : in    std_logic; -- reset input

    -- IO signals
    pin_inTTL    : in    std_logic_vector (4 downto 0); -- TTL inputs
    pin_outTTL   : out   std_logic_vector (4 downto 0); -- TTL outputs
    pin_outPECL  : out   std_logic_vector (4 downto 0); -- LVPECL outputs

    -- Other signals
    pin_RST_BRD  : out   std_logic; -- Board reset output
    pin_LED      : out   std_logic_vector (1 downto 0); -- LEDs ; 0 = red, 1 = green
    pin_EXTS     : out   std_logic_vector (7 downto 0); -- Test header
    pin_SW       : in    std_logic_vector (5 downto 0)  -- Configuration dip switch
  );
end entity top;

--------------------------------------------------------------------------------
------------------------------- ARCHITECTURE -----------------------------------
--------------------------------------------------------------------------------
architecture rtl_top of top is
  ------------------------------------------------------------------------------
  -- constant
  ------------------------------------------------------------------------------

  ------------------------------------------------------------------------------
  -- signal
  ------------------------------------------------------------------------------
  -- Registers
  signal s_cmdReg     : std_logic_vector (31 downto 0); -- command register
  signal s_status     : std_logic_vector (29 downto 0); -- bit 31 & 30 are reserved for access error
  signal s_wRegister0 : std_logic_vector (31 downto 0); -- wRegister 0 ; not used read only register
                                                         -- (time between 2 pulses)
  signal s_dupMissingPulseDelay     : std_logic_vector (31 downto 0); -- maximum delay before a missing
                                                                      -- synchro pulse is signaled
  signal s_holdOffMaxDelay          : std_logic_vector (31 downto 0); -- maximum delay before a missing
                                                                      -- beam is signaled
  signal s_holdOnMaxDelay           : std_logic_vector (31 downto 0); -- maximum delay before after a missing
                                                                      -- beam a re-injection is signaled
  signal s_topUpGateHoldOff         : std_logic_vector (31 downto 0); -- delay before a gating signal is
                                                                      -- triggered after the pre top-up
                                                                      -- signal is received
  signal s_topUpGateWidth           : std_logic_vector (31 downto 0); -- width of the gating windows
  signal s_linacMPMissingPulseDelay : std_logic_vector (31 downto 0); -- maximum delay before a missing
  signal s_wRegister6      : std_logic_vector (31 downto 0); -- wRegister 6 ; not used read only register
                                                         -- (time between 2 pulses)
  signal s_wRegister8      : std_logic_vector (31 downto 0); -- wRegister 8
  signal s_wRegister9      : std_logic_vector (31 downto 0); -- wRegister 9
  signal s_wRegister10     : std_logic_vector (31 downto 0); -- wRegister 10
  signal s_wRegister11     : std_logic_vector (31 downto 0); -- wRegister 11
  signal s_wRegister12     : std_logic_vector (31 downto 0); -- wRegister 12
  signal s_wRegister13     : std_logic_vector (31 downto 0); -- wRegister 13
  signal s_dupPulseMon     : std_logic_vector (31 downto 0); -- time between 2 pulses
  signal s_linacMPPulseMon : std_logic_vector (31 downto 0); -- time between 2 pulses

  signal s_csw       : std_logic_vector (14 downto 0); -- chip select of internal write transfert
  signal s_csr       : std_logic_vector (14 downto 0); -- chip select of internal read transfert
  signal s_clk60MHz  : std_logic; -- 60 MHz clock
  signal s_clk1kHz   : std_logic; -- 1 kHz clock
  signal s_clk500mhz : std_logic; -- 0.5 Hz clock
  signal s_reset     : std_logic;

  -- Duplication signals
  signal s_dupOutTTL  : std_logic_vector(4 downto 0);
  signal s_dupOutPECL : std_logic_vector(4 downto 0);
  signal s_dupLED     : std_logic_vector(1 downto 0);

  -- Topup signals
  signal s_topUpOutTTL  : std_logic_vector(4 downto 0);
  signal s_topUpOutPECL : std_logic_vector(4 downto 0);
  signal s_topUpLED     : std_logic_vector(1 downto 0);
  signal s_topUpError   : std_logic;

  -- LINAC multipulse signals
  signal s_linacMPOutTTL  : std_logic_vector(4 downto 0);
  signal s_linacMPOutPECL : std_logic_vector(4 downto 0);
  signal s_linacMPLED     : std_logic_vector(1 downto 0);

  -- Topup signals
  signal s_testOutTTL  : std_logic_vector(4 downto 0);
  signal s_testOutPECL : std_logic_vector(4 downto 0);
  signal s_testLED     : std_logic_vector(1 downto 0);

  ------------------------------------------------------------------------------
  -- Attributes
  ------------------------------------------------------------------------------
  attribute PERIOD : string;
  attribute PERIOD of s_clk60MHz : signal is "16 ns";

  attribute BUFFER_TYPE : string;
  attribute BUFFER_TYPE of s_clk60MHz : signal is "BUFG";

  ------------------------------------------------------------------------------
  --------------------------------- Main ---------------------------------------
  ------------------------------------------------------------------------------
  begin

  -- IBUFG: Global Clock Buffer (sourced by an external pin)
  --        Spartan-6
  -- Xilinx HDL Language Template, version 13.2
  IBUFG_inst : IBUFG
    generic map (
      IBUF_LOW_PWR => FALSE, -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD => "LVTTL")
    port map (
      O => s_clk60MHz,   -- Clock buffer output
      I => pin_CLK60MHz  -- Clock buffer input (connect directly to top-level port)
    );
  -- End of IBUFG_inst instantiation

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  -- Reset block
  reset : entity work.reset (rtl_reset)
    port map (
      -- Internal bus
      p_ext_RESETn => pin_RESETn,   -- board reset in
      p_clk60MHz   => s_clk60MHz,   -- clock 60 MHz
      p_cmd_REG0   => s_cmdReg(0),  -- command register bit 0
      p_CSW        => s_csw(0),     -- chip select of internal write transfert
      -- Other signals
      p_fpga_RST   => s_reset,      -- reset out
      p_board_RST  => pin_RST_BRD   -- board reset out
    );

  -- Clock generator
  clk_gen : entity work.clk_sources (rtl_clk_sources)
    port map (
      p_clk60MHz  => s_clk60mHz,  -- clock 60 MHz
      p_reset     => s_reset,     -- reset in
      p_clk1kHz   => s_clk1kHz,   -- clock 1 KHz
      p_clk500mHz => s_clk500mhz  -- clock 0.5 Hz
    );

  -- cPCI interface
  -- 0  : 0x00 : board ID (R) => in Hemes module
  -- 1  : 0x04 : firmware version (R) => in Hemes module
  -- 2  : 0x08 : test register (R/W) => in Hemes module
  -- 3  : 0x0C : status register (R/W)
  -- 4  : 0x10 : command register (R/W)
  -- 5  : 0x14 : time between two synchronization pulses (R) [duplication mode]
  -- 6  : 0x18 : maximum delay before a missing synchro pulse is signaled (R/W) [duplication mode]
  -- 7  : 0x1C : maximum delay before a missing beam is signa (R/W) [top-up mode]
  -- 8  : 0x20 : maximum delay  before after a missing beam a re-injection is signaled (R/W) [top-up mode]
  -- 9  : 0x24 : delay before a gating signal is triggered after the pre top-up signal is received (R/W) [top-up]
  -- 10 : 0x28 : width of the gating windows (R/W) [top-up mode]
  -- 11 : 0x2C : time between two synchronization pulses (R) [LINAC multipulse mode]
  -- 12 : 0x30 : maximum delay before a missing synchro pulse is signaled (R/W) [LINAC multipulse mode]
  cPCI_interface : entity work.cPCI_hermes (rtl_cPCI_hermes)
    port map (
      -- PLX bus
      p_AD          => pin_AD,        -- adress bus
      p_DATA        => pin_DATA,      -- data bus
      p_CSn         => pin_CSn,       -- chip select
      p_BEn         => pin_BEn,       -- byte enable
      p_BLASTn      => pin_BLASTn,    -- burst last
      p_LWRn        => pin_LWRn,      -- read/write
      p_RDn         => pin_RDn,       -- read strobe
      p_WRn         => pin_WRn,       -- write strobe
      p_WAITOn      => pin_WAITOn,    -- wait
      p_ADSn        => pin_ADSn,      -- address strobe
      p_ALE         => pin_ALE,       -- address latch
      p_CLK60MHz    => s_clk60MHz,    -- clock 60 MHz
      p_READYn      => pin_READYn,    -- ready
      p_BTERMn      => pin_BTERMn,    -- burst treminate
      p_LINTI       => pin_LINTI,     -- interrupt
      -- Write registers
      p_cmdREG      => s_cmdReg,      -- command register
      p_status_reg  => s_status,      -- status register ; bit 31 & 30 are reserved for access error
      p_wRegister0  => s_wRegister0,  -- wRegister 0 ; not used read only register (time between 2 pulses)
      p_wRegister1  => s_dupMissingPulseDelay, -- maximum delay before a missing synchro pulse is signaled
      p_wRegister2  => s_holdOffMaxDelay,  -- maximum delay before a missing beam is signaled
      p_wRegister3  => s_holdOnMaxDelay,   -- maximum delay before after a missing beam a re-injection is signaled
      p_wRegister4  => s_topUpGateHoldOff, -- delay before a gating signal is triggered after the
                                           -- pre top-up signal is received
      p_wRegister5  => s_topUpGateWidth,   -- width of the gating windows
      p_wRegister6  => s_wRegister6,       -- wRegister 6  ; not used read only register (time between 2 pulses)
      p_wRegister7  => s_linacMPMissingPulseDelay, -- maximum delay before a missing synchro pulse is signaled
      p_wRegister8  => s_wRegister8,  -- wRegister 8  ; not used
      p_wRegister9  => s_wRegister9,  -- wRegister 9  ; not used
      p_wRegister10 => s_wRegister10, -- wRegister 10 ; not used
      p_wRegister11 => s_wRegister11, -- wRegister 11 ; not used
      p_wRegister12 => s_wRegister12, -- wRegister 12 ; not used
      p_wRegister13 => s_wRegister13, -- wRegister 13 ; not used
      -- Read registers
      p_rRegister0  => s_dupPulseMon,              -- time between two synchronization pulses
      p_rRegister1  => s_dupMissingPulseDelay,     -- read back s_dupMissingPulseDelay value
      p_rRegister2  => s_holdOffMaxDelay,          -- read back s_holdOffMaxDelay value
      p_rRegister3  => s_holdOnMaxDelay,           -- read back s_holdOnMaxDelay value
      p_rRegister4  => s_topUpGateHoldOff,         -- read back s_topUpGateHoldOff value
      p_rRegister5  => s_topUpGateWidth,           -- read back s_topUpGateWidth value
      p_rRegister6  => s_linacMPPulseMon,          -- time between two synchronization pulses
      p_rRegister7  => s_linacMPMissingPulseDelay, -- read back s_linacMPMissingPulseDelay value
      p_rRegister8  => s_wRegister8,  -- read back s_wRegister8  value
      p_rRegister9  => s_wRegister9,  -- read back s_wRegister9  value
      p_rRegister10 => s_wRegister10, -- read back s_wRegister10 value
      p_rRegister11 => s_wRegister11, -- read back s_wRegister11 value
      p_rRegister12 => s_wRegister12, -- read back s_wRegister12 value
      p_rRegister13 => s_wRegister13, -- read back s_wRegister13 value
      -- Internal bus
      p_CSW         => s_csw,         -- chip select of internal write transfert
      p_CSR         => s_csr,         -- chip select of internal read transfert
      -- Other signals
      p_reset       => s_reset        -- reset
    );

  -- Status register manager
  -- Status(0) set : time between 2 synchro pulses exceeded the maximum delay [duplication mode]
  -- Status(1) set : time between pre top-up and post top-up exceed maximum   [top-up gating mode]
  -- Status(2) set : time between post top-up and pre top-up exceed maximum   [top-up gating mode]
  -- Status(3) set : time between 2 synchro pulses exceeded the maximum delay [LINAC multipulse mode]
  cPCI_status: entity work.cPCI_statusManager (rtl_cPCI_statusManager)
    port map (
      p_inStates(0) => s_dupLED(0),  -- time between 2 synchro pulses exceeded the maximum delay
      p_inStates(1) => s_topUpError, -- beam lost gated
      p_inStates(2) => s_linacMPLED(0), -- time between 2 synchro pulses exceeded the maximum delay
      p_inStates(29 downto 3) => "000000000000000000000000000", -- not used
      p_status      => s_status,
      -- Other signals
      p_clk60MHz    => s_clk60MHz, -- clock 60 MHz
      p_reset       => s_reset     -- reset
    );

  -- Signal duplication block
  dup : entity work.dup_top (rtl_dup_top)
    port map (
      p_dup_inTTL             => pin_inTTL(0),
      p_dup_outTTL            => s_dupOutTTL,
      p_dup_outPECL           => s_dupOutPECL,
      p_dup_clk1kHz           => s_clk1kHz,
      p_dup_clk500mHz         => s_clk500mhz,
      p_dup_clk60MHz          => s_clk60MHz,
      p_dup_missingPulseDelay => s_dupMissingPulseDelay, -- maximum delay before a missing synchro
                                                         -- pulse is signaled
      p_dup_csw               => s_csw(2),
      p_dup_reset             => s_reset,
      p_dup_pulseMon          => s_dupPulseMon, -- time between two synchronization pulses
      p_dup_LED               => s_dupLED
    );

  -- TopUp gating
  topUp : entity work.topup_top (rtl_topup_top)
    port map (
      p_topUp_inTTL(0)          => pin_inTTL(0),
      p_topUp_inTTL(1)          => pin_inTTL(1),
      p_topUp_outTTL            => s_topUpOutTTL,
      p_topUp_outPECL           => s_topUpOutPECL,
      p_topUp_clk1kHz           => s_clk1kHz,
      p_topUp_clk500mHz         => s_clk500mHz,
      p_topUp_clk60MHz          => s_clk60MHz,
      p_topUp_maxPostTopUpDelay => s_holdOffMaxDelay,
      p_topUp_maxPreTopUpDelay  => s_holdOnMaxDelay,
      p_topUp_gateHoldOff       => s_topUpGateHoldOff,
      p_topUp_gateWidth         => s_topUpGateWidth,
      p_topUp_csw               => s_csw(6 downto 3),
      p_topUp_reset             => s_reset,
      p_topup_Error             => s_topUpError,
      p_topUp_LED               => s_topUpLED
    );

  -- LINAC multipulse block
  LinacMP : entity work.linacMP_top (rtl_linacMP_top)
    port map (
      p_linacMP_inTTL             => pin_inTTL(3 downto 0),
      p_linacMP_outTTL            => s_linacMPOutTTL,
      p_linacMP_outPECL           => s_linacMPOutPECL,
      p_linacMP_clk1kHz           => s_clk1kHz,
      p_linacMP_clk500mHz         => s_clk500mhz,
      p_linacMP_clk60MHz          => s_clk60MHz,
      p_linacMP_missingPulseDelay => s_linacMPMissingPulseDelay, -- maximum delay before a
                                                                 --  missing synchro pulse is signaled
      p_linacMP_csw               => s_csw(7),
      p_linacMP_reset             => s_reset,
      p_linacMP_pulseMon          => s_linacMPPulseMon, -- time between two synchronization pulses
      p_linacMP_LED               => s_linacMPLED
    );

  -- Test
  test : entity work.test_top (rtl_test_top)
    port map (
      p_test_outTTL    => s_testOutTTL,
      p_test_outPECL   => s_testOutPECL,
      p_test_CLK500mHz => s_clk500mHz,
      p_test_reset     => s_reset,
      p_test_LED       => s_testLED
    );

  -- Multiplexer
  outMux : entity work.outputMux (rtl_outputMux)
    port map (
      -- Duplication block inputs
      p_dup_outTTL      => s_dupOutTTL,
      p_dup_outPECL     => s_dupOutPECL,
      p_dup_LED         => s_dupLED,
      -- Top-up block inputs
      p_topUp_outTTL    => s_topUpOutTTL,
      p_topUp_outPECL   => s_topUpOutPECL,
      p_topUp_LED       => s_topUpLED,
      -- LINAC multipulse block inputs
      p_linacMP_outTTL  => s_linacMPOutTTL,
      p_linacMP_outPECL => s_linacMPOutPECL,
      p_linacMP_LED     => s_linacMPLED,
      -- Test block inputs
      p_test_outTTL     => s_testOutTTL,
      p_test_outPECL    => s_testOutPECL,
      p_test_LED        => s_testLED,
       -- Mux block outputs
      p_mux_SWsel       => pin_SW,
      p_mux_outTTL      => pin_outTTL,
      p_mux_outPECL     => pin_outPECL,
      p_mux_outLED      => pin_LED
    );

  ------------------------------------------------------------------------------
  ------------------------------------------------------------------------------
  pin_EXTS(7 downto 0) <= "00000000";

end architecture rtl_top;
